#include "gzfstream.h"


#ifdef HAVE_LIBZ 
 
#include <string.h>

void ogzfstream::open ( const char* path, 
		       int level = Z_DEFAULT_COMPRESSION,
		       int strategy = Z_DEFAULT_STRATEGY )
{
  ostrstream mode_str;
  
  mode_str << "wb" << level;
 
  if ( strategy == Z_FILTERED )
    mode_str << "f";
  else if ( strategy == Z_HUFFMAN_ONLY )
    mode_str << "h";

  mode_str << ends;

  __my_file = gzopen ( path, mode_str.str () );
}

void ogzfstream::open ( int fd,
			int level = Z_DEFAULT_COMPRESSION,
			int strategy = Z_DEFAULT_STRATEGY )
{
  ostrstream mode_str;
  
  mode_str << "wb" << level;

  if ( strategy == Z_FILTERED )
    mode_str << "f";
  else if ( strategy == Z_HUFFMAN_ONLY )
    mode_str << "h";

  mode_str << ends;

  __my_file = gzdopen ( fd, mode_str.str () );
}

int ogzfstream::setParams ( int level = Z_DEFAULT_COMPRESSION,
			    int strategy = Z_DEFAULT_STRATEGY )
{
  return gzsetparams ( __my_file, level, strategy );
}
 
z_off_t ogzfstream::seek ( z_off_t offset, int whence )
{
  flush ();
  return gzseek ( __my_file, offset, whence );
}

z_off_t ogzfstream::tell ()
{
  return gztell ( __my_file );
}

void ogzfstream::close ()
{
  flush ();
  gzclose ( __my_file );
  __my_file = NULL;
}

int ogzfstream::flush ( int flush_type )
{
  return gzflush ( __my_file, flush_type );
}


void ogzfstream::flush ()
{
  *this << ends;
  gzwrite ( __my_file, (void*)__buffer.str (), strlen ( __buffer.str () ) );
  __buffer.freeze ( false );
}

ostream& ogzfstream::write ( const char* str , streamsize n )
{
  flush ();
  gzwrite ( __my_file, (void*) str, n );
  return *this;
}


#endif
