/* -*- Mode: C++ -*- */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#include <string.h>
#include <string>
#include <stdlib.h>
#include "paramreader.h"

ParamReader::ParamReader()
{
}

ParamReader::~ParamReader()
{
}


void ParamReader::readParams ( const char* path_name )
{
  string path = tildeExpand ( path_name );
  
  ifstream ifile;

    ifile.open ( path.c_str () );
    if ( !ifile.is_open () )
	{
	  ofstream ofile;
	  ofile.open ( path.c_str () );
	  if ( !ofile.is_open () )
	    {
	      cerr << "can't open config file, " << path << endl ;
	    }
	  else
	    {
	      writeConfig ( ofile );
	      ofile.close ();
	      cout << "created config file, " << path << endl;
	      return;
	    } 
	}

    int line_no = 1;
    while( ifile.good () ) //!ifile.eof () )
      {
	/* ignore remark line */	
	if ( ifile.peek () == '#' || ifile.peek () == '\n')
	  {
	    // ignore this line
	    strstreambuf tmp;
	    ifile.get ( tmp, '\n' ); 
	    // get the '\n' that the get above, didn't get
	    ifile.ignore ();
	    line_no++;
	    continue ;
	  }
	
	string param;
	ifile >> param;
	if ( param.empty () )
	  {
	    continue;
	  }


	bool find_colon = true;
	if ( param[ param.length () - 1 ] == ':' )
	  {
	    param.resize ( param.length () - 1 );
	    find_colon = false;
	  }

	bool match = true;
	map < string, int* >::iterator int_i;
	map < string, bool* >::iterator bool_i;
	map < string, string* >::iterator str_i;
	map < string, double* >::iterator double_i;

	if ( ( bool_i = bool_map.find ( param ) ) != bool_map.end () )
	  {
	    *(bool_i->second) = true;
	  }
	else
	  {
	   if ( find_colon )
	     {
	       string colon;
	       ifile >> colon;
	       if ( colon != string ( ":" ) )
		 {
		   cerr << path << ": " << line_no 
			<< ": Expecting colon after parameter name, not `" 
			<< colon << "'\n";
	    
		   // ignore the rest of this line
		   strstreambuf tmp;
		   ifile.get ( tmp, '\n' ); 
		   // get the `\n' that the get above, didn't get
		   ifile.ignore ();
		   line_no++;
		   continue;
		 }
	     } 
	   
	   if ( ( int_i = int_map.find ( param ) ) != int_map.end () )
	     {
	       ifile >> *(int_i->second);
	     }
	   else if ( ( bool_i = onoff_map.find ( param ) ) != onoff_map.end () )
	     {
	       string value;
	       ifile >> value;
	       if ( value == "on" || value == "true" )
		 *(bool_i->second) = true;
	       else if ( value == "off" || value == "false" )
		 *(bool_i->second) = false;
	       else
		 {
		   cerr << path << ": " << line_no 
			<< ": Expecting `on', `off', `true' or `false', not `" 
			<< value << "'\n";
	    
		   // ignore the rest of this line
		   strstreambuf tmp;
		   ifile.get ( tmp, '\n' ); 
		   // get the '\n' that the get above, didn't get
		   ifile.ignore ();
		   line_no++;
		   continue;
		 }
	     }
	   else if ( ( str_i = str_map.find ( param ) ) != str_map.end () )
	     {
	       ifile >> *(str_i->second);
	     }
	   else if ( ( double_i = double_map.find ( param ) ) 
		     != double_map.end () )
	     {
	       ifile >> *(double_i->second);
	     }
	   else
	     match = false;
	  }
	

	if ( ifile.fail () )
	  {
	    string value;
	    ifile.clear ();
	    ifile >> value;
	    cerr << path << ": " << line_no 
		 << ": Error converting value `"
		 << value << "'\n";
	  }
	
	if (!match)
	  {
	    cerr << path << ": " << line_no 
		 << ": Unrecognized parameter in file `"
		 << param << "'\n";
	  }

	// ignore the rest of this line
	strstreambuf tmp;
	ifile.get ( tmp, '\n' ); 
	// get the '\n' that the get above, didn't get
	ifile.ignore ();
	line_no++;
      }

    ifile.close () ;
}


void ParamReader::readOptions ( const int& orig_argc, 
				const char* const orig_argv[] )
{
    int p ;
    int argc = orig_argc;
    const char * const *argv = orig_argv;
    char buf[MaxMesg];
    
    argc--;
    argv++;
    
    while (argc)
	{
	if ( sscanf ( *argv, "-%s", buf ) != 1 )
	    {
	    argv++ ;
	    argc-- ;
	    continue;
	    }

	string param = buf;
	bool match = true;
	map < string, int* >::iterator int_i;
	map < string, bool* >::iterator bool_i;
	map < string, string* >::iterator str_i;
	map < string, double* >::iterator double_i;

	if ( ( bool_i = bool_map.find ( param ) ) != bool_map.end () )
	  {
	    *(bool_i->second) = true;
	  }
	else
	  {
	    istrstream ifile ( *argv + 1 );
	    if ( ( int_i = int_map.find ( param ) ) != int_map.end () )
	      {
		ifile >> *(int_i->second);
	      } 
	    else if ( ( bool_i = onoff_map.find ( param ) ) != onoff_map.end () )
	      {
		string value;
		ifile >> value;
		if ( value == "on" || value == "true" )
		  *(bool_i->second) = true;
		else if ( value == "off" || value == "false" )
		  *(bool_i->second) = false;
		else
		  {
		    cerr << __FILE__ << ": " << __LINE__ 
			 << ": Expecting `on', `off', `true' or `false', not `" 
			 << value << "' for command line option `" << param 
			 << "'\n";
		    match = false;
		  }
	      }
	    else if ( ( str_i = str_map.find ( param ) ) != str_map.end () )
	      {
		ifile >> *(str_i->second);
	      }
	    else if ( ( double_i = double_map.find ( param ) ) 
		      != double_map.end () )
	      {
		ifile >> *(double_i->second);
	      }
	    else
	      match = false;
	
	    if ( match )
	      {
		argv++ ;
		argc-- ;
	      }
	  }
	if (!match) {
	  /* pfr: It's "okay" for options to be unrecognized on the command line because
	     it may be intended for the another parameter reader instantiation */
	  //cerr << "Unrecognized parameter on command line: " << buf << endl;
	}

	argv++ ;
	argc-- ;
	}

}
