/* -*- Mode: c++ -*- */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#ifndef _PARAMREADER_H_
#define _PARAMREADER_H_

//#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <strstream>
#include <string>
#include <map>
#include "param.h"
#include "types.h"
#include "utility.h"

using namespace std;

class ParamReader
{
public:

  enum param_type
  {
    INT,
    STRING,
    DOUBLE,
    BOOL,
    ONOFF
  };

  struct param_t {
    string	optname ;
    union
    {
      string*	str_p;
      int*	int_p;
      bool*	bool_p;
      double*	double_p;
    };
    ParamReader::param_type	vsize ;
  };


  ParamReader();
  virtual ~ParamReader();

protected:
  void
  readParams ( const char* path );
  void 
  readOptions ( const int& argc, 
		const char* const *argv );
    
  virtual void getOptions( const int& argc, const char * const *argv ) = 0;
  virtual void writeConfig ( ostream& o ) = 0;

  map < string, int* > 		int_map;
  map < string, string* > 	str_map;
  map < string, bool* > 	bool_map;
  map < string, bool* > 	onoff_map;
  map < string, double* > 	double_map;

  void add2Maps ( const string& key, int* value )
  {
    int_map[key] = value;
  }

  void add2Maps ( const string& key, bool* value, bool onoff = false )
  {
    if ( onoff )
      onoff_map[key] = value;
    else
      bool_map[key] = value;
  }

  void add2Maps ( const string& key, string* value )
  {
    str_map[key] = value;
  }

  void add2Maps ( const string& key, double* value )
  {
    double_map[key] = value;
  }


};


#endif
