/* -*- Mode: C++ -*- */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#include <string>
#include "config.h"

#include "playerparam.h"


const char PlayerParam::PLAYER_CONF[] = "~/.rcssserver-player.conf";

const int PlayerParam::DEFAULT_PLAYER_TYPES = 7;
const int PlayerParam::DEFAULT_SUBS_MAX = 3;
const int PlayerParam::DEFAULT_PT_MAX = 3;

const double PlayerParam::DEFAULT_PLAYER_SPEED_MAX_DELTA_MIN = 0.0;
const double PlayerParam::DEFAULT_PLAYER_SPEED_MAX_DELTA_MAX = 0.2;
const double PlayerParam::DEFAULT_STAMINA_INC_MAX_DELTA_FACTOR = -100.0;
    
const double PlayerParam::DEFAULT_PLAYER_DECAY_DELTA_MIN = 0.0;
const double PlayerParam::DEFAULT_PLAYER_DECAY_DELTA_MAX = 0.2;
const double PlayerParam::DEFAULT_INERTIA_MOMENT_DELTA_FACTOR = 25.0;

const double PlayerParam::DEFAULT_DASH_POWER_RATE_DELTA_MIN = 0.0;
const double PlayerParam::DEFAULT_DASH_POWER_RATE_DELTA_MAX = 0.002;
const double PlayerParam::DEFAULT_PLAYER_SIZE_DELTA_FACTOR = -100.0;

const double PlayerParam::DEFAULT_KICKABLE_MARGIN_DELTA_MIN = 0.0;
const double PlayerParam::DEFAULT_KICKABLE_MARGIN_DELTA_MAX = 0.2;
const double PlayerParam::DEFAULT_KICK_RAND_DELTA_FACTOR = 0.5;

const double PlayerParam::DEFAULT_EXTRA_STAMINA_DELTA_MIN = 0.0;
const double PlayerParam::DEFAULT_EXTRA_STAMINA_DELTA_MAX = 100.0;
const double PlayerParam::DEFAULT_EFFORT_MAX_DELTA_FACTOR = -0.002;
const double PlayerParam::DEFAULT_EFFORT_MIN_DELTA_FACTOR = -0.002;

const int    PlayerParam::DEFAULT_RANDOM_SEED = -1; //negative means generate a new seed

PlayerParam::PlayerParam(const int& argc, const char * const *argv ) 
{ 
  getOptions ( argc, argv ) ;
}

PlayerParam::~PlayerParam()
{
}

void PlayerParam::getOptions(const int& argc, const char * const *argv )
{
  add2Maps ( "player_types", &player_types );
  add2Maps ( "subs_max", &subs_max );
  add2Maps ( "pt_max", &pt_max );
  add2Maps ( "player_speed_max_delta_min", &player_speed_max_delta_min );
  add2Maps ( "player_speed_max_delta_max", &player_speed_max_delta_max );
  add2Maps ( "stamina_inc_max_delta_factor", &stamina_inc_max_delta_factor );
  add2Maps ( "player_decay_delta_min", &player_decay_delta_min );
  add2Maps ( "player_decay_delta_max", &player_decay_delta_max );
  add2Maps ( "inertia_moment_delta_factor", &inertia_moment_delta_factor );
  add2Maps ( "dash_power_rate_delta_min", &dash_power_rate_delta_min );
  add2Maps ( "dash_power_rate_delta_max", &dash_power_rate_delta_max );
  add2Maps ( "player_size_delta_factor", &player_size_delta_factor );
  add2Maps ( "kickable_margin_delta_min", &kickable_margin_delta_min );
  add2Maps ( "kickable_margin_delta_max", &kickable_margin_delta_max );
  add2Maps ( "kick_rand_delta_factor", &kick_rand_delta_factor );
  add2Maps ( "extra_stamina_delta_min", &extra_stamina_delta_min );
  add2Maps ( "extra_stamina_delta_max", &extra_stamina_delta_max );
  add2Maps ( "effort_max_delta_factor", &effort_max_delta_factor );
  add2Maps ( "effort_min_delta_factor", &effort_min_delta_factor );
  add2Maps ( "random_seed", &random_seed );

  player_types = PlayerParam::DEFAULT_PLAYER_TYPES;
  subs_max = PlayerParam::DEFAULT_SUBS_MAX;
  pt_max = PlayerParam::DEFAULT_PT_MAX;
    
  player_speed_max_delta_min = PlayerParam::DEFAULT_PLAYER_SPEED_MAX_DELTA_MIN;
  player_speed_max_delta_max = PlayerParam::DEFAULT_PLAYER_SPEED_MAX_DELTA_MAX;
  stamina_inc_max_delta_factor = PlayerParam::DEFAULT_STAMINA_INC_MAX_DELTA_FACTOR;
    
  player_decay_delta_min = PlayerParam::DEFAULT_PLAYER_DECAY_DELTA_MIN;
  player_decay_delta_max = PlayerParam::DEFAULT_PLAYER_DECAY_DELTA_MAX;
  inertia_moment_delta_factor = PlayerParam::DEFAULT_INERTIA_MOMENT_DELTA_FACTOR;

  dash_power_rate_delta_min = PlayerParam::DEFAULT_DASH_POWER_RATE_DELTA_MIN;
  dash_power_rate_delta_max = PlayerParam::DEFAULT_DASH_POWER_RATE_DELTA_MAX;
  player_size_delta_factor = PlayerParam::DEFAULT_PLAYER_SIZE_DELTA_FACTOR;

  kickable_margin_delta_min = PlayerParam::DEFAULT_KICKABLE_MARGIN_DELTA_MIN;
  kickable_margin_delta_max = PlayerParam::DEFAULT_KICKABLE_MARGIN_DELTA_MAX;
  kick_rand_delta_factor = PlayerParam::DEFAULT_KICK_RAND_DELTA_FACTOR;

  extra_stamina_delta_min = PlayerParam::DEFAULT_EXTRA_STAMINA_DELTA_MIN;
  extra_stamina_delta_max = PlayerParam::DEFAULT_EXTRA_STAMINA_DELTA_MAX;
  effort_max_delta_factor = PlayerParam::DEFAULT_EFFORT_MAX_DELTA_FACTOR;
  effort_min_delta_factor = PlayerParam::DEFAULT_EFFORT_MIN_DELTA_FACTOR;

  random_seed = PlayerParam::DEFAULT_RANDOM_SEED;
    
  bool file_specified = false;

  for(int i = 1 ; i < argc ; i++)
    {
      if ( strcmp( argv [ i ], "-pfile" ) == 0)
	{
	  file_specified = true;
	  i++;
	  readParams ( argv [ i ] );
	}
    }
    
  if (!file_specified)
    readParams ( PlayerParam::PLAYER_CONF );
     
  readOptions ( argc, argv );

  if ( player_types < 1 )
    {
      cerr << "Error: must have at least one player type\n";
      exit ( EXIT_FAILURE );
    }
    
}

void PlayerParam::writeConfig ( ostream& o )
{
  o << "# Configurations for soccerserver players\n";
  o << "# Lines that start '#' are comment lines.\n";
  o << endl;
  o << "# number of player types including default player type\n";
  o << "player_types: " << player_types << endl;
  o << endl;
  o << "# number of substitutions allowed during game\n";
  o << "subs_max: " << subs_max << endl;
  o << endl;
  o << "# min and max deltas for adjusting player_speed_max\n";
  o << "player_speed_max_delta_min: " << player_speed_max_delta_min << endl;
  o << "player_speed_max_delta_max: " << player_speed_max_delta_max << endl;
  o << endl;
  o << "# amount delta is multiplied by for stamina_inc_max\n";
  o << "stamina_inc_max_delta_factor: " << stamina_inc_max_delta_factor << endl;
  o << endl;
  o << "# min and max deltas for adjusting player_decay\n";
  o << "player_decay_delta_min: " << player_decay_delta_min << endl;
  o << "player_decay_delta_max: " << player_decay_delta_max << endl;
  o << endl;
  o << "# amount delta is multiplied by for inertia_moment\n";
  o << "inertia_moment_delta_factor: " << inertia_moment_delta_factor << endl;
  o << endl;
  o << "# min and max deltas for adjusting dash_power_rate\n";
  o << "dash_power_rate_delta_min: " << dash_power_rate_delta_min << endl;
  o << "dash_power_rate_delta_max: " << dash_power_rate_delta_max << endl;
  o << endl;
  o << "# amount delta is multiplied by for player_size\n";
  o << "player_size_delta_factor: " << player_size_delta_factor << endl;
  o << endl;
  o << "# min and max deltas for adjusting kickable_margin\n";
  o << "kickable_margin_delta_min: " << kickable_margin_delta_min << endl;
  o << "kickable_margin_delta_max: " << kickable_margin_delta_max << endl;
  o << endl;
  o << "# amount delta is multiplied by for kick_rand\n";
  o << "kick_rand_delta_factor: " << kick_rand_delta_factor << endl;
  o << endl;
  o << "# min and max deltas for adjusting extra_stamina\n";
  o << "extra_stamina_delta_min: " << extra_stamina_delta_min << endl;
  o << "extra_stamina_delta_max: " << extra_stamina_delta_max << endl;
  o << endl;
  o << "# amount delta is multiplied by for effort_max\n";
  o << "effort_max_delta_factor: " << effort_max_delta_factor << endl;
  o << endl;
  o << "# amount delta is multiplied by for effort_min\n";
  o << "effort_min_delta_factor: " << effort_min_delta_factor << endl;
  o << endl;
}


player_params_t PlayerParam::convertToStruct ()
{
  player_params_t tmp;

  tmp.player_types = htons( (short) player_types );
  tmp.subs_max = htons( (short) subs_max );
  tmp.pt_max = htons( (short) pt_max );
    
  tmp.player_speed_max_delta_min = htonl( (long)rint((SHOWINFO_SCALE2 * player_speed_max_delta_min) ));
  tmp.player_speed_max_delta_max = htonl( (long)rint((SHOWINFO_SCALE2 * player_speed_max_delta_max) ));
  tmp.stamina_inc_max_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * stamina_inc_max_delta_factor) ));
    
  tmp.player_decay_delta_min = htonl( (long)rint((SHOWINFO_SCALE2 * player_decay_delta_min) ));
  tmp.player_decay_delta_max = htonl( (long)rint((SHOWINFO_SCALE2 * player_decay_delta_max) ));
  tmp.inertia_moment_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * inertia_moment_delta_factor) ));

  tmp.dash_power_rate_delta_min = htonl( (long)rint((SHOWINFO_SCALE2 * dash_power_rate_delta_min) ));
  tmp.dash_power_rate_delta_max = htonl( (long)rint((SHOWINFO_SCALE2 * dash_power_rate_delta_max) ));
  tmp.player_size_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * player_size_delta_factor) ));

  tmp.kickable_margin_delta_min = htonl( (long)rint((SHOWINFO_SCALE2 * kickable_margin_delta_min) ));
  tmp.kickable_margin_delta_max = htonl( (long)rint((SHOWINFO_SCALE2 * kickable_margin_delta_max) ));
  tmp.kick_rand_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * kick_rand_delta_factor) ));

  tmp.extra_stamina_delta_min = htonl( (long)rint((SHOWINFO_SCALE2 * extra_stamina_delta_min) ));
  tmp.extra_stamina_delta_max = htonl( (long)rint((SHOWINFO_SCALE2 * extra_stamina_delta_max) ));
  tmp.effort_max_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * effort_max_delta_factor) ));
  tmp.effort_min_delta_factor = htonl( (long)rint((SHOWINFO_SCALE2 * effort_min_delta_factor) ));
  tmp.random_seed = htonl( (long)random_seed );
    
  return tmp;
}


void PlayerParam::getSettingsString ( char* buf, const double& ver )
{
  ostrstream ost(buf, MaxStringSize) ;

  if ( ver >= 8.0 )
    {
      ost << "(player_param "
	  << "(player_types " << player_types
	  << ") (subs_max " << subs_max 
	  << ") (pt_max " << pt_max 
	  << ") (player_speed_max_delta_min " << player_speed_max_delta_min
	  << ") (player_speed_max_delta_max " << player_speed_max_delta_max
	  << ") (stamina_inc_max_delta_factor " << stamina_inc_max_delta_factor
	  << ") (player_decay_delta_min " << player_decay_delta_min
	  << ") (player_decay_delta_max " << player_decay_delta_max
	  << ") (inertia_moment_delta_factor " << inertia_moment_delta_factor
	  << ") (dash_power_rate_delta_min " << dash_power_rate_delta_min
	  << ") (dash_power_rate_delta_max " << dash_power_rate_delta_max
	  << ") (player_size_delta_factor " << player_size_delta_factor
	  << ") (kickable_margin_delta_min " << kickable_margin_delta_min
	  << ") (kickable_margin_delta_max " << kickable_margin_delta_max
	  << ") (kick_rand_delta_factor " << kick_rand_delta_factor
	  << ") (extra_stamina_delta_min " << extra_stamina_delta_min
	  << ") (extra_stamina_delta_max " << extra_stamina_delta_max
	  << ") (effort_max_delta_factor " << effort_max_delta_factor
	  << ") (effort_min_delta_factor " << effort_min_delta_factor
	  << ") (random_seed " << random_seed
	  << "))" << endl << ends;
    }
  else
    {
      ost << "(player_param "
	  << player_types << " "
	  << subs_max << " "
	  << pt_max << " "
    
	  << player_speed_max_delta_min << " "
	  << player_speed_max_delta_max << " "
	  << stamina_inc_max_delta_factor << " "
    
	  << player_decay_delta_min << " "
	  << player_decay_delta_max << " "
	  << inertia_moment_delta_factor << " "

	  << dash_power_rate_delta_min << " "
	  << dash_power_rate_delta_max << " "
	  << player_size_delta_factor << " "

	  << kickable_margin_delta_min << " "
	  << kickable_margin_delta_max << " "
	  << kick_rand_delta_factor << " "

	  << extra_stamina_delta_min << " "
	  << extra_stamina_delta_max << " "
	  << effort_max_delta_factor << " "
	  << effort_min_delta_factor
	  << ")" << endl << ends;
    }
}
