/* -*- Mode: C -*-
 *Header:
 *File: replay.C
 *Author: Noda Itsuki
 *Date: 1996/10/07
 *EndHeader:
 */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */


#include <math.h>
#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include <strstream.h>

#include "param.h"
#include "types.h"
#include "utility.h"
#include "object.h"
#include "field.h"


/*
 *===================================================================
 *Part: Replay Loop
 *===================================================================
 */
void Stadium::ReplayLoop() 
{
	void replay_sig_check(void) ;
	struct itimerval itv ;
	struct sigaction alarm_action ;

	int sent = 0, rect = 0 ;
	FILE *fp ;
	dispinfo_t buf ;
	displist *p ;

	itv.it_interval.tv_sec = 0 ;
	itv.it_interval.tv_usec = TIMEDELTA * 1000 ;
	itv.it_value.tv_sec = 0 ;
	itv.it_value.tv_usec = TIMEDELTA * 1000 ;

#ifdef Linux_2_2	
	alarm_action.sa_handler = (void (*)(int))replay_sig_check ;
#else
	alarm_action.sa_handler = (void (*)(...))replay_sig_check ;
#endif	
	alarm_action.sa_flags = 0; // [2000/11/20.frehberg.cs.tu-berlin.de]

	sigaction(SIGALRM, &alarm_action, NULL) ;

	setitimer(ITIMER_REAL, &itv, NULL) ;

	if ((fp = fopen(server_param->replay.c_str (), "rb")) == NULL) {
		cerr << "Can't open replay file " << server_param->replay << endl ;
		return ;
	}

	for (;;) {
		sigpause(SIGUSR1) ;

		sent += TIMEDELTA ;
		rect += TIMEDELTA ;

		if (sent >= server_param->send_st ) {
			if (top.next != NULL) {
				if (fread((char *)(&buf), sizeof(buf), 1, fp) == 0) {
					fclose(fp) ;
					break ;
				}
				for ( p = top.next ; p != NULL ; p = p->next)
					udp_send(&buf, p->shost, p->sport, 0) ;
			}
			sent = 0 ;
		}
		if (rect >= server_param->recv_st) {
			udp_recv_message() ;
			rect = 0 ;
		}
	}
	closeLogs ();
	return ;
}

void replay_sig_check(void)
{
	/* This function does nothing. */
}

