/* -*- Mode: c++ -*- */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */

#include <string.h>

#include "serverparam.h"
#include <string>
#include "config.h"

#define name_of(var)	(#var)
#define str_of(var)	(string(#var))

const string ServerParam::LANDMARK_FILE = "~/.rcssserver-landmark.xml";
const string ServerParam::SERVER_CONF = "~/.rcssserver-server.conf";

const int ServerParam::SEND_COMMS = true;

const int ServerParam::TEXT_LOGGING = true;
const int ServerParam::GAME_LOGGING = true;
const int ServerParam::GAME_LOG_VERSION = 3;
const string ServerParam::TEXT_LOG_DIR = "./";
const string ServerParam::GAME_LOG_DIR = "./";
const string ServerParam::TEXT_LOG_FIXED_NAME = "rcssserver";
const string ServerParam::GAME_LOG_FIXED_NAME = "rcssserver";
const int ServerParam::TEXT_LOG_FIXED = false;
const int ServerParam::GAME_LOG_FIXED = false;
const int ServerParam::TEXT_LOG_DATED = true;
const int ServerParam::GAME_LOG_DATED = true;
const string ServerParam::LOG_DATE_FORMAT = "%Y%m%d%H%M-";
const int ServerParam::LOG_TIMES = false;
const int ServerParam::RECORD_MESSAGES = true;
const int ServerParam::TEXT_LOG_COMPRESSION = 0;
const int ServerParam::GAME_LOG_COMPRESSION = 0;


ServerParam::ServerParam(const int& argc, const char * const *argv ) 
{ 
  getOptions ( argc, argv ) ;
}

ServerParam::~ServerParam()
{
}

void ServerParam::getOptions(const int& argc, const char * const *argv )
{
  add2Maps ( "goal_width", &gwidth ); 
  add2Maps ( "player_size", &psize ); 
  add2Maps ( "player_decay", &pdecay ); 
  add2Maps ( "player_rand", &prand ); 
  add2Maps ( "player_weight", &pweight ); 
  add2Maps ( "player_speed_max", &pspeed_max ); 
  // th 6.3.00
  add2Maps ( "player_accel_max", &paccel_max ); 
  //
  add2Maps ( "stamina_max", &stamina_max ); 
  add2Maps ( "stamina_inc_max", &stamina_inc ); 
  add2Maps ( "recover_dec_thr", &recover_dthr ); 
  add2Maps ( "recover_min", &recover_min ); 
  add2Maps ( "recover_dec", &recover_dec ); 
  add2Maps ( "effort_dec_thr", &effort_dthr ); 
  add2Maps ( "effort_min", &effort_min ); 
  add2Maps ( "effort_dec", &effort_dec ); 
  add2Maps ( "effort_inc_thr", &effort_ithr ); 
  add2Maps ( "effort_inc", &effort_inc ); 
  // pfr 8/14/00: for RC2000 evaluation
  add2Maps ( "kick_rand",           &kick_rand ); 
  add2Maps ( "team_actuator_noise", &team_actuator_noise, true ); 
  add2Maps ( "prand_factor_l",      &prand_factor_l ); 
  add2Maps ( "prand_factor_r",      &prand_factor_r ); 
  add2Maps ( "kick_rand_factor_l",  &kick_rand_factor_l ); 
  add2Maps ( "kick_rand_factor_r",  &kick_rand_factor_r ); 

  add2Maps ( "ball_size", &bsize ); 
  add2Maps ( "ball_decay", &bdecay ); 
  add2Maps ( "ball_rand", &brand ); 
  add2Maps ( "ball_weight", &bweight ); 
  add2Maps ( "ball_speed_max", &bspeed_max ); 
  // th 6.3.00
  add2Maps ( "ball_accel_max", &baccel_max ); 
  //
  add2Maps ( "dash_power_rate", &dprate ); 
  add2Maps ( "kick_power_rate", &kprate ); 
  add2Maps ( "kickable_margin", &kmargin ); 
  add2Maps ( "control_radius", &ctlradius ); 

  add2Maps ( "catch_probability", &catch_prob ); 
  add2Maps ( "catchable_area_l",    &catch_area_l ); 
  add2Maps ( "catchable_area_w",    &catch_area_w ); 
  add2Maps ( "goalie_max_moves",    &goalie_max_moves ); 

  //already below
  //add2Maps ( "wind_dir", &windir ); 
  add2Maps ( "maxpower", &maxp ); 
  add2Maps ( "minpower", &minp ); 
  add2Maps ( "maxmoment", &maxm ); 
  add2Maps ( "minmoment", &minm ); 
  add2Maps ( "maxneckmoment", &maxnm ); 
  add2Maps ( "minneckmoment", &minnm ); 
  add2Maps ( "maxneckang", &maxn ); 
  add2Maps ( "minneckang", &minn ); 
  add2Maps ( "visible_angle", &visangle ); 
  add2Maps ( "visible_distance", &visdist ); 
  add2Maps ( "audio_cut_dist", &audio_dist ); 
  add2Maps ( "quantize_step", &dist_qstep ); 
  add2Maps ( "quantize_step_l", &land_qstep ); 
#ifdef NEW_QSTEP
  add2Maps ( "quantize_step_dir", &dir_qstep ); 
  add2Maps ( "quantize_step_dist_team_l", &dist_qstep_l ); 
  add2Maps ( "quantize_step_dist_team_r", &dist_qstep_r ); 
  add2Maps ( "quantize_step_dist_l_team_l", &land_qstep_l ); 
  add2Maps ( "quantize_step_dist_l_team_r", &land_qstep_r ); 
  add2Maps ( "quantize_step_dir_team_l", &dir_qstep_l ); 
  add2Maps ( "quantize_step_dir_team_r", &dir_qstep_r ); 
#endif
  add2Maps ( "ckick_margin", &ckmargin ); 
  add2Maps ( "wind_dir", &windir ); 
  add2Maps ( "wind_force", &winforce ); 
  add2Maps ( "wind_rand", &winrand ); 
  add2Maps ( "inertia_moment", &inertia_moment ); 
  add2Maps ( "wind_none", &win_no ); 
  add2Maps ( "wind_random", &win_random ); 
  add2Maps ( "half_time", &half_time ); 
  add2Maps ( "drop_ball_time", &drop_time ); 
  add2Maps ( "port", &portnum ); 
  add2Maps ( "coach_port", &coach_pnum ); 
  add2Maps ( "olcoach_port", &olcoach_pnum ); 
  add2Maps ( "say_coach_cnt_max", &say_cnt_max ); 
  add2Maps ( "say_coach_msg_size", &SayCoachMsgSize ); 
  add2Maps ( "simulator_step", &sim_st ); 
  add2Maps ( "send_step", &send_st ); 
  add2Maps ( "recv_step", &recv_st ); 
  add2Maps ( "sense_body_step", &sb_step ); 
  add2Maps ( "say_msg_size", &SayMsgSize ); 
  add2Maps ( "clang_win_size",      &clang_win_size ); 
  add2Maps ( "clang_define_win",    &clang_define_win ); 
  add2Maps ( "clang_meta_win",      &clang_meta_win ); 
  add2Maps ( "clang_advice_win",    &clang_advice_win ); 
  add2Maps ( "clang_info_win",      &clang_info_win ); 
  add2Maps ( "clang_mess_delay",    &clang_mess_delay ); 
  add2Maps ( "clang_mess_per_cycle",      &clang_mess_per_cycle ); 
  add2Maps ( "hear_max", &hear_max ); 
  add2Maps ( "hear_inc", &hear_inc ); 
  add2Maps ( "hear_decay", &hear_decay ); 
  add2Maps ( "catch_ban_cycle", &cban_cycle ); 
  add2Maps ( "coach", &CoachMode ); 
  add2Maps ( "coach_w_referee", &CwRMode ); 
  add2Maps ( "old_coach_hear", &old_hear ); 
  add2Maps ( "send_vi_step",        &sv_st ); 
  add2Maps ( "use_offside", &useoffside, true ); 
  add2Maps ( "offside_active_area_size", &offside_area ); 
  add2Maps ( "forbid_kick_off_offside", &kickoffoffside, true ); 
  add2Maps ( "verbose", &verbose, true ); 
  add2Maps ( "replay", &replay ); 
  add2Maps ( "offside_kick_margin", &offside_kick_margin ); 
  add2Maps ( "slow_down_factor",        &slow_down_factor ); 
  add2Maps ( "synch_mode",              &synch_mode, true ); //pfr:SYNCH
  add2Maps ( "synch_offset",            &synch_offset ); //pfr:SYNCH
  add2Maps ( "synch_micro_sleep",       &synch_micro_sleep ); //pfr:SYNCH  

  add2Maps ( "start_goal_l",            &start_goal_l ); 
  add2Maps ( "start_goal_r",            &start_goal_r ); 

  add2Maps ( "fullstate_l",             &fullstate_l, true ); 
  add2Maps ( "fullstate_r",             &fullstate_r, true ); 
  add2Maps ( "slowness_on_top_for_left_team", &slowness_on_top_for_left_team ); 
  add2Maps ( "slowness_on_top_for_right_team", &slowness_on_top_for_right_team ); 
  add2Maps ( "landmark_file",           &landmark_file ); 
  add2Maps ( "send_comms", &send_comms, true ); 
  add2Maps ( "text_logging", &text_logging, true ); 
  add2Maps ( "game_logging", &game_logging, true ); 
  add2Maps ( "game_log_version", &game_log_version ); 
  add2Maps ( "text_log_dir", &text_log_dir ); 
  add2Maps ( "game_log_dir", &game_log_dir ); 
  add2Maps ( "text_log_fixed_name", &text_log_fixed_name ); 
  add2Maps ( "game_log_fixed_name", &game_log_fixed_name ); 
  add2Maps ( "text_log_fixed", &text_log_fixed, true ); 
  add2Maps ( "game_log_fixed", &game_log_fixed, true ); 
  add2Maps ( "text_log_dated", &text_log_dated, true ); 
  add2Maps ( "game_log_dated", &game_log_dated, true ); 
  add2Maps ( "log_date_format", &log_date_format ); 
  add2Maps ( "log_times", &log_times, true ); 
  add2Maps ( "record_messages", &record_messages, true ); 
  add2Maps ( str_of ( text_log_compression ), &text_log_compression ); 
  add2Maps ( str_of ( game_log_compression ), &game_log_compression ); 


  /* set default parameter */
  gwidth = GOAL_WIDTH ;
  psize = PLAYER_SIZE ;
  pdecay = PLAYER_DECAY ;
  prand = PLAYER_RAND ;
  pweight = PLAYER_WEIGHT ;
  stamina_max = STAMINA_MAX ;
  stamina_inc = STAMINA_INC_MAX ;
  recover_dthr = RECOVERY_DEC_THR ;
  recover_min = RECOVERY_MIN ;
  recover_dec = RECOVERY_DEC ;
  effort_dthr = EFFORT_DEC_THR ;
  effort_min = EFFORT_MIN ;
  effort_dec = EFFORT_DEC ;
  effort_ithr = EFFORT_INC_THR ;
  effort_inc= EFFORT_INC ;
  inertia_moment = IMPARAM ;
  // pfr 8/14/00: for RC2000 evaluation
  kick_rand = KICK_RAND;
  team_actuator_noise = FALSE;
  prand_factor_l = PRAND_FACTOR_L;
  prand_factor_r = PRAND_FACTOR_R;
  kick_rand_factor_l = KICK_RAND_FACTOR_L;
  kick_rand_factor_r = KICK_RAND_FACTOR_R;

  bsize = BALL_SIZE ;
  bdecay = BALL_DECAY ;
  brand = BALL_RAND ;
  bweight = BALL_WEIGHT ;
  bspeed_max = BALL_SPEED_MAX ;
  pspeed_max = PLAYER_SPEED_MAX ;
  // th 6.3.00
  baccel_max = BALL_ACCEL_MAX ;
  paccel_max = PLAYER_ACCEL_MAX ;
  //
  dprate = DASHPOWERRATE ;
  kprate = KICKPOWERRATE ;
  kmargin = KICKABLE_MARGIN ;
  ctlradius = CONTROL_RADIUS ;
  ckmargin = CORNER_KICK_MARGIN ;
  catch_prob = GOALIE_CATCHABLE_POSSIBILITY ;
  catch_area_l = GOALIE_CATCHABLE_AREA_LENGTH ; 
  catch_area_w = GOALIE_CATCHABLE_AREA_WIDTH ;
  goalie_max_moves = GOALIE_MAX_MOVES;
  maxp = MAXPOWER ;
  minp = MINPOWER ;
  maxm = MAXMOMENT ;
  minm = MINMOMENT ;
  maxnm = MAX_NECK_MOMENT ;
  minnm = MIN_NECK_MOMENT ;
  maxn = MAX_NECK_ANGLE ;
  minn = MIN_NECK_ANGLE ;
  visangle = VisibleAngle ;
  visdist = VisibleDistance ;
  half_time = HALF_TIME ;
  drop_time = DROP_TIME ;
  portnum = DEFAULT_PORT_NUMBER ;
  coach_pnum = COACH_PORT_NUMBER ;
  olcoach_pnum = OLCOACH_PORT_NUMBER ;
  say_cnt_max = DEF_SAY_COACH_CNT_MAX ;
  SayCoachMsgSize = DEF_SAY_COACH_MSG_SIZE ;
  sim_st = SIMULATOR_STEP_INTERVAL_MSEC ;
  send_st = UDP_SEND_STEP_INTERVAL_MSEC ;
  recv_st = UDP_RECV_STEP_INTERVAL_MSEC ;
  sb_step = SENSE_BODY_INTERVAL_MSEC ;
  SayMsgSize = DEF_SAY_MSG_SIZE ;
  clang_win_size = 300;
  clang_define_win = 1;
  clang_meta_win = 1;
  clang_advice_win = 1;
  clang_info_win = 1;
  clang_mess_delay = 50;
  clang_mess_per_cycle = 1;
  CoachMode = FALSE ;
  CwRMode = FALSE ;
  old_hear = FALSE ;
  sv_st = SEND_VISUALINFO_INTERVAL_MSEC ;
  replay[0] = NULLCHAR ;
  audio_dist = AUDIO_CUT_OFF_DIST ;
  hear_max = HEAR_MAX ;
  hear_inc = HEAR_INC ;
  hear_decay = HEAR_DECAY ;
  cban_cycle = GOALIE_CATCH_BAN_CYCLE ;
  slow_down_factor = 1;
  verbose = FALSE ;

  dist_qstep = DIST_QSTEP ;
  land_qstep = LAND_QSTEP ;
#ifdef NEW_QSTEP
  dir_qstep  = DIR_QSTEP ;
#endif

  windir = WIND_DIR ;
  winforce = WIND_FORCE ;
  winrand = WIND_RAND ;
  win_no = FALSE ;
  win_random = FALSE ;
  useoffside = TRUE ;
  offside_area = OFFSIDE_ACTIVE_AREA_SIZE ;
  kickoffoffside = FALSE ; 
  offside_kick_margin = OFFSIDE_KICK_MARGIN ;

//pfr:SYNCH
  synch_mode = false;
  synch_offset = 60;
  synch_micro_sleep = 1;

  start_goal_l = 0;
  start_goal_r = 0;

  fullstate_l = FALSE;
  fullstate_r = FALSE;

  /*  // skip command name */
  /*  int tmp_argc = argc-1; */
  /*  argv++ ; argc-- ; */

#ifdef NEW_QSTEP
  bool defined_qstep_l = false;
  bool defined_qstep_r = false;
  bool defined_qstep_l_l = false;
  bool defined_qstep_l_r = false;
  bool defined_qstep_dir_l = false;
  bool defined_qstep_dir_r = false;
#endif

  slowness_on_top_for_left_team = 1.0;
  slowness_on_top_for_right_team = 1.0;

  landmark_file = ServerParam::LANDMARK_FILE;


  send_comms = ServerParam::SEND_COMMS;
  text_logging = ServerParam::TEXT_LOGGING;
  game_logging = ServerParam::GAME_LOGGING;
  game_log_version = ServerParam::GAME_LOG_VERSION;
  text_log_dir = ServerParam::TEXT_LOG_DIR;
  game_log_dir = ServerParam::GAME_LOG_DIR;
  text_log_fixed_name = ServerParam::TEXT_LOG_FIXED_NAME;
  game_log_fixed_name = ServerParam::GAME_LOG_FIXED_NAME;
  text_log_fixed = ServerParam::TEXT_LOG_FIXED;
  game_log_fixed = ServerParam::GAME_LOG_FIXED;
  text_log_dated = ServerParam::TEXT_LOG_DATED;
  game_log_dated = ServerParam::GAME_LOG_DATED;
  log_date_format = ServerParam::LOG_DATE_FORMAT;
  log_times = ServerParam::LOG_TIMES;
  record_messages = ServerParam::RECORD_MESSAGES;
  text_log_compression = ServerParam::TEXT_LOG_COMPRESSION;
  game_log_compression = ServerParam::GAME_LOG_COMPRESSION;

  /* first, search option '-sfile' */
  int i ;
  bool file_specified = false;

  for(int i = 1 ; i < argc ; i++)
    {
      if ( strcmp( argv [ i ], "-sfile" ) == 0)
	{
	  file_specified = true;
	  i++;
	  readParams ( argv [ i ] );
	}
    }

  if (!file_specified)
    readParams ( ServerParam::SERVER_CONF.c_str () );
  

  readOptions ( argc, argv );

#ifdef NEW_QSTEP
  if ( !defined_qstep_l )
    dist_qstep_l = dist_qstep ;
  if ( !defined_qstep_r )
    dist_qstep_r = dist_qstep ;
  if ( !defined_qstep_l_l )
    land_qstep_l = land_qstep ;
  if ( !defined_qstep_l_r )
    land_qstep_r = land_qstep ;
  if ( !defined_qstep_dir_l )
    dir_qstep_l = dir_qstep ;
  if ( !defined_qstep_dir_r )
    dir_qstep_r = dir_qstep ;
#endif

  visangle = Deg2Rad(visangle) ;
  half_time = half_time * 1000 / sim_st ;
  /*send_st /= 4 ; */
  kickable_area = kmargin + bsize + psize ;
  ctlradius_width = ctlradius - psize ;
  /* apply the slow down factor */
  sim_st  *= slow_down_factor;
  sb_step *= slow_down_factor;
  sv_st   *= slow_down_factor;
  send_st *= slow_down_factor;
  synch_offset *= slow_down_factor; //pfr:SYNCH
  lcm_st = lcm (sim_st, lcm (send_st, lcm (recv_st, lcm (sb_step, sv_st) ) ) );
  lcm_st = lcm (lcm_st, (synch_mode ? synch_offset : 1)); //pfr:SYNCH

  text_log_dir = tildeExpand ( text_log_dir );
  game_log_dir = tildeExpand ( game_log_dir );


}

void ServerParam::writeConfig ( ostream& o )
{
  o << "# Configurations for soccerserver\n";
  o << "# Lines that start '#' are comment lines.\n";
  o << "\n";
  o << "# width of goals\n";
  o << "goal_width: " << gwidth << endl;
  o << "\n";
  o << "# size, decay, random parameter, weight, maximum speed of players\n";
  o << "player_size: " << psize << endl;
  o << "player_decay: " << pdecay << endl;
  o << "player_rand: " << prand << endl;
  o << "player_weight: " << pweight << endl;
  o << "player_speed_max: " << pspeed_max << endl;
  o << "player_accel_max: " << paccel_max << endl;
  o << "\n";
  o << "# maximum and recovery step of player's stamina\n";
  o << "stamina_max: " << stamina_max << endl;
  o << "stamina_inc_max        : " << stamina_inc << endl;
  o << "\n";
  o << "# decriment threshold ,decriment step and minimum of player's recovery\n";
  o << "recover_dec_thr        : " << recover_dthr << endl;
  o << "recover_dec: " << recover_dec << endl;
  o << "recover_min: " << recover_min << endl;
  o << "\n";
  o << "# decriment threshold, decriment step, incriment threshold, incriment step\n";
  o << "# and minimum of player's effort\n";
  o << "effort_dec_thr: " << effort_dthr << endl;
  o << "effort_dec: " << effort_dec << endl;
  o << "effort_inc_thr: " << effort_ithr << endl;
  o << "effort_inc: " << effort_inc << endl;
  o << "effort_min: " << effort_min << endl;
  o << "\n";
  o << "# maximum, recovery step and decay of player's hear capacity\n";
  o << "hear_max: " << hear_max << endl;
  o << "hear_inc: " << hear_inc << endl;
  o << "hear_decay: " << hear_decay << endl;
  o << "\n";
  o << "# inertia moment of player\n";
  o << "inertia_moment: " << inertia_moment << endl;
  o << "\n";
  o << "# interval of sense_body\n";
  o << "sense_body_step: " << sb_step << endl;
  o << "\n";
  o << "# goalie catchable area length, width\n";
  o << "catchable_area_l        : " << catch_area_l << endl;
  o << "catchable_area_w        : " << catch_area_w << endl;
  o << "\n";
  o << "# goalie catchbale probability\n";
  o << "catch_probability       : " << catch_prob << endl;
  o << "\n";
  o << "# goalie catch ban cycle\n";
  o << "catch_ban_cycle         : " << cban_cycle << endl;
  o << "\n";
  o << "# goalie max moves after catch\n";
  o << "goalie_max_moves        : " << goalie_max_moves << endl;
  o << "\n";
  o << "# size, decay, random parameter, weight and maximum speed of a ball\n";
  o << "ball_size: " << bsize << endl;
  o << "ball_decay: " << bdecay << endl;
  o << "ball_rand: " << brand << endl;
  o << "ball_weight: " << bweight << endl;
  o << "ball_speed_max: " << bspeed_max << endl;
  o << "ball_accel_max: " << baccel_max << endl;
  o << "\n";
  o << "# force, direction and random parameter of wind\n";
  o << "wind_force: " << winforce << endl;
  o << "wind_dir: " << windir << endl;
  o << "wind_rand: " << winrand << endl;
  o << "\n";
  o << "# kickable margin. kickable_area = kickable_margin + bsize + psize\n";
  o << "kickable_margin        : " << kmargin << endl;
  o << "\n";
  o << "# corner kick margin\n";
  o << "ckick_margin: " << ckmargin << endl;
  o << "\n";
  o << "# magnification of power in dash, kick \n";
  o << "dash_power_rate        : " << dprate << endl;
  o << "kick_power_rate        : " << kprate << endl;
  o << "\n";
  o << "# kick randomeness\n";
  o << "kick_rand: " << kick_rand << endl;
  o << "\n";
  o << "# angle of view corn [unit: degree]\n";
  o << "visible_angle        : " << visangle << endl;
  o << "\n";
  o << "# audio cut off distance\n";
  o << "audio_cut_dist        : " << audio_dist << endl;
  o << "\n";
  o << "# quantize step of distance for move_object, landmark\n";
  o << "quantize_step: " << dist_qstep << endl;
  o << "quantize_step_l        : " << land_qstep << endl;
  o << "\n";
  o << "# max and min of power in dash and kick\n";
  o << "maxpower: " << maxp << endl;
  o << "minpower: " << minp << endl;
  o << "\n";
  o << "# max and min of power in turn\n";
  o << "maxmoment: " << maxm << endl;
  o << "minmoment: " << minm << endl;
  o << "\n";
  o << "# max and min of power in turn_neck\n";
  o << "maxneckmoment: " << maxnm << endl;
  o << "minneckmoment: " << minnm << endl;
  o << "\n";
  o << "# max and min of neck angle\n";
  o << "maxneckang: " << maxn << endl;
  o << "minneckang: " << minn << endl;
  o << "\n";
  o << "# Default port number\n";
  o << "port: " << portnum << endl;
  o << "\n";
  o << "# Default coach port number\n";
  o << "coach_port: " << coach_pnum << endl;
  o << "\n";
  o << "# Default oline coach port number\n";
  o << "olcoach_port: " << olcoach_pnum << endl;
  o << "\n";
  o << "# Default upper limit of the number of online coach's message\n";
  o << "say_coach_cnt_max: " << say_cnt_max << endl;
  o << "\n";
  o << "# Default upper limit of length of online coach's message\n";
  o << "say_coach_msg_size: " << SayCoachMsgSize << endl;
  o << "\n";
  o << "# Parameters for the new coach language\n";
  o << "# time wondow which controls how many messages can be sent\n";
  o << "clang_win_size          : " << clang_win_size << endl;
  o << "# number of messages per window\n";
  o << "clang_define_win        : " << clang_define_win << endl;
  o << "clang_meta_win          : " << clang_meta_win << endl;
  o << "clang_advice_win        : " << clang_advice_win << endl;
  o << "clang_info_win          : " << clang_info_win << endl;
  o << "# delay between receipt of message and send to players\n";
  o << "clang_mess_delay        : " << clang_mess_delay << endl;
  o << "# maximum number of coach messages sent per cycle\n";
  o << "clang_mess_per_cycle    : " << clang_mess_per_cycle << endl;
  o << "\n";
  o << "# Default interval of online coach's look\n";
  o << "send_vi_step        : " << sv_st << endl;
  o << "\n";
  o << "# time step of simulation [unit:msec]\n";
  o << "simulator_step        : " << sim_st << endl;
  o << "\n";
  o << "# time step of visual information [unit:msec]\n";
  o << "send_step: " << send_st << endl;
  o << "\n";
  o << "# time step of acception of command [unit: msec]\n";
  o << "recv_step: " << recv_st << endl;
  o << "\n";
  o << "# length of half of game [unit:sec]\n";
  o << "# (if negative, a game does not stop automatically)\n";
  o << "half_time: " << half_time << endl;
  o << "\n";
  o << "# number of cycles to wait until dropping the ball automatically\n";
  o << "# for free kicks, corner kicks and so on\n";
  o << "# (0 means do not drop automatically)\n";
  o << "drop_ball_time        : " << drop_time << endl;
  o << "\n";
  o << "# string size of say message [unit:byte]\n";
  o << "say_msg_size        : " << SayMsgSize << endl;
  o << "\n";
  o << "# flag for using off side rule. [on/off]\n";
  o << "use_offside: " << ( useoffside ? "on" : "off" ) << endl;
  o << "\n";
  o << "# offside active area size\n";
  o << "offside_active_area_size: " << offside_area << endl;
  o << "\n";
  o << "# forbid kick off offside. [on/off] \n";
  o << "forbid_kick_off_offside: " << ( kickoffoffside ? "on" : "off" ) << endl;
  o << "\n";
  o << "# flag for verbose mode. [on/off]\n";
  o << "verbose: " << ( verbose ? "on" : "off" ) << endl;
  o << "\n";
  o << "# offside kick margin. \n";
  o << "offside_kick_margin: " << offside_kick_margin << endl;
  o << "\n";
  o << "\n";
  o << "#coach_w_referee\n";
  o << "\n";
  o << "# how much to slow a player down when it's on the top half of the field\n";
  o << "#\n";
  o << "# 1.0 means no change\n";
  o << "# 2.0 means twice as slow, 10.0 means ten times as slow, etc\n";
  o << "# 0.5 means twice as fast\n";
  o << "# 0.0 and less is invalid.  This is not error checked\n";
  o << "slowness_on_top_for_left_team  : " 
    << slowness_on_top_for_left_team << endl;
  o << "slowness_on_top_for_right_team : " 
    << slowness_on_top_for_right_team << endl;
  o << "\n";
  o << "synch_mode              : " << ( synch_mode ? "on" : "off" ) << endl;
  o << "#synch_micro_sleep      : " << synch_micro_sleep << endl;
  o << "\n";
  o << "# Send client commands to monitors. [on/off]\n";
  o << "send_comms: " << ( send_comms ? "on" : "off" ) << endl;
  o << "\n";
  o << "# LOGGING\n";
  o << "# game logging. [on/off]\n";
  o << "text_logging: " << ( text_logging ? "on" : "off" ) << endl;
  o << "game_logging: " << ( game_logging ? "on" : "off" ) << endl;
  o << "# game_log version.\n";
  o << "game_log_version        : " << game_log_version << endl;
  o << "# Output directories\n";
  o << "text_log_dir            : " << text_log_dir << endl;
  o << "game_log_dir            : " << game_log_dir << endl;
  o << "# Fixed output names\n";
  o << "text_log_fixed_name     : " << text_log_fixed_name << endl;
  o << "game_log_fixed_name     : " << game_log_fixed_name << endl;
  o << "# Use fixed output names\n";
  o << "text_log_fixed          : " << ( text_log_fixed ? "on\n" : "off\n" );
  o << "game_log_fixed          : " << ( game_log_fixed ? "on\n" : "off\n" );
  o << "# Add date stamp to output names\n";
  o << "text_log_dated          : " << ( text_log_dated ? "on\n" : "off\n" );
  o << "game_log_dated          : " << ( game_log_dated ? "on\n" : "off\n" );
  o << "log_date_format         : " << log_date_format << endl;
  o << "# Write ms between cycles in text log. [on/off]\n";
  o << "log_times: " << ( log_times ? "on\n" : "off\n" );
  o << "# Record messages in game log\n";
  o << "record_messages         : " << ( record_messages ? "on\n" : "off\n" );
  o << "# Log file compression\n";
  o << "text_log_compression          : " << text_log_compression << endl;
  o << "game_log_compression          : " << game_log_compression << endl;
  o << "\n";
}



server_params_t ServerParam::convertToStruct ()
{
  server_params_t tmp;

  tmp.gwidth = htonl( (long)(SHOWINFO_SCALE2 * gwidth) );
  tmp.inertia_moment = htonl( (long)(SHOWINFO_SCALE2 * inertia_moment) );
  tmp.psize = htonl( (long)(SHOWINFO_SCALE2 * psize) );
  tmp.pdecay = htonl( (long)(SHOWINFO_SCALE2 * pdecay) );
  tmp.prand = htonl( (long)(SHOWINFO_SCALE2 * prand) );
  tmp.pweight = htonl( (long)(SHOWINFO_SCALE2 * pweight) );
  tmp.pspeed_max = htonl( (long)(SHOWINFO_SCALE2 * pspeed_max) );
  tmp.paccel_max = htonl( (long)(SHOWINFO_SCALE2 * paccel_max) );
  tmp.stamina_max = htonl( (long)(SHOWINFO_SCALE2 * stamina_max) );
  tmp.stamina_inc = htonl( (long)(SHOWINFO_SCALE2 * stamina_inc) );
  tmp.recover_init = htonl( (long)(SHOWINFO_SCALE2 * recover_init) );
  tmp.recover_dthr = htonl( (long)(SHOWINFO_SCALE2 * recover_dthr) );
  tmp.recover_min = htonl( (long)(SHOWINFO_SCALE2 * recover_min) );
  tmp.recover_dec = htonl( (long)(SHOWINFO_SCALE2 * recover_dec) );
  tmp.effort_init = htonl( (long)(SHOWINFO_SCALE2 * effort_init) );
  tmp.effort_dthr = htonl( (long)(SHOWINFO_SCALE2 * effort_dthr) );
  tmp.effort_min = htonl( (long)(SHOWINFO_SCALE2 * effort_min) );
  tmp.effort_dec = htonl( (long)(SHOWINFO_SCALE2 * effort_dec) );
  tmp.effort_ithr = htonl( (long)(SHOWINFO_SCALE2 * effort_ithr) );
  tmp.effort_inc = htonl( (long)(SHOWINFO_SCALE2 * effort_inc) );
  tmp.kick_rand = htonl( (long)(SHOWINFO_SCALE2 * kick_rand) );                                
  tmp.team_actuator_noise = htons( (short) team_actuator_noise );                        
  tmp.prand_factor_l = htonl( (long)(SHOWINFO_SCALE2 * prand_factor_l ) );                           
  tmp.prand_factor_r = htonl( (long)(SHOWINFO_SCALE2 * prand_factor_r ) );                           
  tmp.kick_rand_factor_l = htonl( (long)(SHOWINFO_SCALE2 * kick_rand_factor_l ) );                       
  tmp.kick_rand_factor_r = htonl( (long)(SHOWINFO_SCALE2 * kick_rand_factor_r ) );                       
  tmp.bsize = htonl( (long)(SHOWINFO_SCALE2 * bsize) );
  tmp.bdecay = htonl( (long)(SHOWINFO_SCALE2 * bdecay) );
  tmp.brand = htonl( (long)(SHOWINFO_SCALE2 * brand) );
  tmp.bweight = htonl( (long)(SHOWINFO_SCALE2 * bweight) );
  tmp.bspeed_max = htonl( (long)(SHOWINFO_SCALE2 * bspeed_max) );
  tmp.baccel_max = htonl( (long)(SHOWINFO_SCALE2 * baccel_max ) );
  tmp.dprate = htonl( (long)(SHOWINFO_SCALE2 * dprate) );
  tmp.kprate = htonl( (long)(SHOWINFO_SCALE2 * kprate) );
  tmp.kmargin = htonl( (long)(SHOWINFO_SCALE2 * kmargin) );
  tmp.ctlradius = htonl( (long)(SHOWINFO_SCALE2 * ctlradius) );
  tmp.ctlradius_width = htonl( (long)(SHOWINFO_SCALE2 * ctlradius_width) );
  tmp.maxp = htonl( (long)(SHOWINFO_SCALE2 * maxp) );
  tmp.minp = htonl( (long)(SHOWINFO_SCALE2 * minp) );
  tmp.maxm = htonl( (long)(SHOWINFO_SCALE2 * maxm) );
  tmp.minm = htonl( (long)(SHOWINFO_SCALE2 * minm) );
  tmp.maxnm = htonl( (long)(SHOWINFO_SCALE2 * maxnm) );
  tmp.minnm = htonl( (long)(SHOWINFO_SCALE2 * minnm) );
  tmp.maxn = htonl( (long)(SHOWINFO_SCALE2 * maxn) );
  tmp.minn = htonl( (long)(SHOWINFO_SCALE2 * minn) );
  tmp.visangle = htonl( (long)(SHOWINFO_SCALE2 * visangle) );
  tmp.visdist = htonl( (long)(SHOWINFO_SCALE2 * visdist) );
  tmp.windir = htonl( (long)(SHOWINFO_SCALE2 * windir) );
  tmp.winforce = htonl( (long)(SHOWINFO_SCALE2 * winforce) );
  tmp.winang = htonl( (long)(SHOWINFO_SCALE2 * winang) );
  tmp.winrand = htonl( (long)(SHOWINFO_SCALE2 * winrand) );
  tmp.kickable_area = htonl( (long)(SHOWINFO_SCALE2 * kickable_area) );
  tmp.catch_area_l = htonl( (long)(SHOWINFO_SCALE2 * catch_area_l) );
  tmp.catch_area_w = htonl( (long)(SHOWINFO_SCALE2 * catch_area_w) );
  tmp.catch_prob = htonl( (long)(SHOWINFO_SCALE2 * catch_prob) );
  tmp.goalie_max_moves = htons( (short) goalie_max_moves );                 
  tmp.ckmargin = htonl( (long)(SHOWINFO_SCALE2 * ckmargin) );
  tmp.offside_area = htonl( (long)(SHOWINFO_SCALE2 * offside_area) );
  tmp.win_no = htons( (short) win_no);
  tmp.win_random = htons( (short) win_random) ;
  tmp.say_cnt_max = htons( (short) say_cnt_max) ;
  tmp.SayCoachMsgSize = htons( (short) SayCoachMsgSize) ;
  tmp.clang_win_size = htons( (short) clang_win_size );
  tmp.clang_define_win = htons( (short) clang_define_win );
  tmp.clang_meta_win = htons( (short) clang_meta_win );
  tmp.clang_advice_win = htons( (short) clang_advice_win );
  tmp.clang_info_win = htons( (short) clang_info_win );
  tmp.clang_mess_delay = htons( (short) clang_mess_delay );
  tmp.clang_mess_per_cycle = htons( (short) clang_mess_per_cycle );
  tmp.half_time = htons( (short) half_time) ;
  tmp.sim_st = htons( (short) sim_st) ;
  tmp.send_st = htons( (short) send_st) ;
  tmp.recv_st = htons( (short) recv_st) ;
  tmp.sb_step = htons( (short) sb_step) ;
  tmp.lcm_st = htons( (short) lcm_st) ;                        
  tmp.SayMsgSize = htons( (short) SayMsgSize) ;
  tmp.hear_max = htons( (short) hear_max) ;
  tmp.hear_inc = htons( (short) hear_inc) ;
  tmp.hear_decay = htons( (short) hear_decay) ;
  tmp.cban_cycle = htons( (short) cban_cycle) ;
  tmp.slow_down_factor = htons( (short) slow_down_factor) ;                          
  tmp.useoffside = htons( (short) useoffside) ;
  tmp.kickoffoffside = htons( (short) kickoffoffside) ;
  tmp.offside_kick_margin = htonl( (long)(SHOWINFO_SCALE2 * offside_kick_margin) );
  tmp.audio_dist = htonl( (long)(SHOWINFO_SCALE2 * audio_dist) );
  tmp.dist_qstep = htonl( (long)(SHOWINFO_SCALE2 * dist_qstep) );
  tmp.land_qstep = htonl( (long)(SHOWINFO_SCALE2 * land_qstep) );
#ifdef NEW_QSTEP
  tmp.dir_qstep = htonl( (long)(SHOWINFO_SCALE2 * dir_qstep) );
  tmp.dist_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * dist_qstep_l) );
  tmp.dist_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * dist_qstep_r) );
  tmp.land_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * land_qstep_l) );
  tmp.land_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * land_qstep_r) );
  tmp.dir_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * dir_qstep_l) );
  tmp.dir_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * dir_qstep_r) );
#else
  tmp.dir_qstep = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.dist_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.dist_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.land_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.land_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.dir_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  tmp.dir_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
#endif    
  tmp.CoachMode = htons( (short) CoachMode) ;
  tmp.CwRMode = htons( (short) CwRMode) ;
  tmp.old_hear = htons( (short) old_hear) ;
  tmp.sv_st = htons( (short) sv_st) ;

  tmp.synch_mode   = htons( (short)synch_mode);//pfr:SYNCH
  tmp.synch_offset = htons( (short)synch_offset);//pfr:SYNCH
  tmp.synch_micro_sleep = htons( (short)synch_micro_sleep);//pfr:SYNCH

  tmp.start_goal_l = htons( (short)start_goal_l );
  tmp.start_goal_r = htons( (short)start_goal_r );

  tmp.fullstate_l = htons( (short)fullstate_l );
  tmp.fullstate_r = htons( (short)fullstate_r );

  tmp.drop_time = htons( (short) drop_time) ;

  tmp.slowness_on_top_for_left_team = htonl( (long)(SHOWINFO_SCALE2 * slowness_on_top_for_left_team) );
  tmp.slowness_on_top_for_right_team = htonl( (long)(SHOWINFO_SCALE2 * slowness_on_top_for_left_team) );
  return tmp;
}


void ServerParam::getSettingsString ( char* buf, const double& ver )
{
  ostrstream ost(buf, MaxStringSize) ;

  if ( ver >= 8.0 )
    {
      ost << "(server_param "
	  << "(goal_width " << gwidth 
	  << ") (player_size "  << psize
	  << ") (player_decay " << pdecay
	  << ") (player_rand " << prand
	  << ") (player_weight " << pweight
	  << ") (player_speed_max " << pspeed_max
	  << ") (player_accel_max " << paccel_max
	  << ") (stamina_max " << stamina_max
	  << ") (stamina_inc_max " << stamina_inc
	  << ") (recover_dec_thr " << recover_dthr
	  << ") (recover_min " << recover_min 
	  << ") (recover_dec " << recover_dec 
	  << ") (effort_dec_thr " << effort_dthr
	  << ") (effort_min " << effort_min
	  << ") (effort_dec " << effort_dec
	  << ") (effort_inc_thr " << effort_ithr
	  << ") (effort_inc " << effort_inc
	  << ") (kick_rand " << kick_rand           
	  << ") (team_actuator_noise " << team_actuator_noise 
	  << ") (prand_factor_l " << prand_factor_l      
	  << ") (prand_factor_r " << prand_factor_r      
	  << ") (kick_rand_factor_l " << kick_rand_factor_l  
	  << ") (kick_rand_factor_r " << kick_rand_factor_r  
	  << ") (ball_size " << bsize
	  << ") (ball_decay " << bdecay
	  << ") (ball_rand " << brand
	  << ") (ball_weight " << bweight
	  << ") (ball_speed_max " << bspeed_max
	  << ") (ball_accel_max " << baccel_max
	  << ") (dash_power_rate " << dprate
	  << ") (kick_power_rate " <<kprate
	  << ") (kickable_margin " <<kmargin
	  << ") (control_radius " << ctlradius
	  << ") (catch_probability " << catch_prob
	  << ") (catchable_area_l " << catch_area_l
	  << ") (catchable_area_w " << catch_area_w
	  << ") (goalie_max_moves " << goalie_max_moves 
	  << ") (maxpower " << maxp
	  << ") (minpower " << minp
	  << ") (maxmoment " << maxm
	  << ") (minmoment " <<minm
	  << ") (maxneckmoment " <<maxnm
	  << ") (minneckmoment " <<minnm
	  << ") (maxneckang " << maxn
	  << ") (minneckang " << minn
	  << ") (visible_angle " <<visangle
	  << ") (visible_distance " << visdist
	  << ") (audio_cut_dist " << audio_dist
	  << ") (quantize_step " << dist_qstep
	  << ") (quantize_step_l " << land_qstep
#ifdef NEW_QSTEP
	  << ") (quantize_step_dir " << dir_qstep
	  << ") (quantize_step_dist_team_l " << dist_qstep_l
	  << ") (quantize_step_dist_team_r " << dist_qstep_r
	  << ") (quantize_step_dist_l_team_l " << land_qstep_l
	  << ") (quantize_step_dist_l_team_r " << land_qstep_r
	  << ") (quantize_step_dir_team_l " << dir_qstep_l
	  << ") (quantize_step_dir_team_r " << dir_qstep_r
#else
	  << ") (quantize_step_dir " <<-1
	  << ") (quantize_step_dist_team_l " <<-1
	  << ") (quantize_step_dist_team_r " <<-1
	  << ") (quantize_step_dist_l_team_l " << -1
	  << ") (quantize_step_dist_l_team_r " << -1
	  << ") (quantize_step_dir_team_l " << -1
	  << ") (quantize_step_dir_team_r " << -1
#endif
	  << ") (ckick_margin " << ckmargin
	  << ") (wind_dir " << windir
	  << ") (wind_force " << winforce
	  << ") (wind_rand " <<winrand
	  << ") (inertia_moment " << inertia_moment
	  << ") (wind_none " <<win_no
	  << ") (wind_random " <<win_random
	  << ") (half_time " <<half_time
	  << ") (drop_ball_time " << drop_time
	  << ") (port " << portnum
	  << ") (coach_port " << coach_pnum
	  << ") (olcoach_port " << olcoach_pnum
	  << ") (say_coach_cnt_max " << say_cnt_max
	  << ") (say_coach_msg_size " << SayCoachMsgSize
	  << ") (simulator_step " << sim_st
	  << ") (send_step " <<send_st
	  << ") (recv_step " <<recv_st
	  << ") (sense_body_step " <<sb_step
	  << ") (say_msg_size " << SayMsgSize
	  << ") (clang_win_size " << clang_win_size
	  << ") (clang_define_win " << clang_define_win
	  << ") (clang_meta_win " << clang_meta_win
	  << ") (clang_advice_win " << clang_advice_win
	  << ") (clang_info_win " << clang_info_win
	  << ") (clang_mess_delay " << clang_mess_delay
	  << ") (clang_mess_per_cycle " << clang_mess_per_cycle
	  << ") (hear_max " << hear_max
	  << ") (hear_inc " << hear_inc
	  << ") (hear_decay " << hear_decay
	  << ") (catch_ban_cycle " << cban_cycle
	  << ") (coach " <<CoachMode
	  << ") (coach_w_referee " << CwRMode
	  << ") (old_coach_hear " << old_hear
	  << ") (send_vi_step " << sv_st
	  << ") (use_offside " << useoffside
	  << ") (offside_active_area_size " << offside_area
	  << ") (forbid_kick_off_offside " << kickoffoffside
	  << ") (verbose " << verbose
	  << ") (replay " << replay
	  << ") (offside_kick_margin " << offside_kick_margin
	  << ") (slow_down_factor " << slow_down_factor   
	  << ") (synch_mode " << synch_mode
	  << ") (synch_offset " << synch_offset
	  << ") (synch_micro_sleep " << synch_micro_sleep
	  << ") (start_goal_l " << start_goal_l
	  << ") (start_goal_r " << start_goal_r
	  << ") (fullstate_l " << fullstate_l
	  << ") (fullstate_r " << fullstate_r
	  << ") (slowness_on_top_for_left_team " 
	  << slowness_on_top_for_left_team 
	  << ") (slowness_on_top_for_right_team " 
	  << slowness_on_top_for_right_team  
	  << ") (" << name_of( send_comms ) << " " << send_comms      
	  << ") (" << name_of( text_logging ) << " " << text_logging
	  << ") (" << name_of( game_logging ) << " " << game_logging
	  << ") (" << name_of( game_log_version ) << " " << game_log_version
	  << ") (" << name_of( text_log_dir ) << " " << text_log_dir
	  << ") (" << name_of( game_log_dir ) << " " << game_log_dir
	  << ") (" << name_of( text_log_fixed_name ) << " " 
	  << text_log_fixed_name
	  << ") (" << name_of( game_log_fixed_name ) << " " 
	  << game_log_fixed_name
	  << ") (" << name_of( text_log_fixed ) << " " << text_log_fixed
	  << ") (" << name_of( game_log_fixed ) << " " << game_log_fixed
	  << ") (" << name_of( text_log_dated ) << " " << text_log_dated
	  << ") (" << name_of( game_log_dated ) << " " << game_log_dated
	  << ") (" << name_of( log_date_format ) << " " << log_date_format
	  << ") (" << name_of( log_times ) << " " << log_times
	  << ") (" << name_of( record_messages ) << " " << record_messages
	  << ") (" << name_of( text_log_compression ) << " " 
	  << text_log_compression
	  << ") (" << name_of( game_log_compression ) << " " 
	  << game_log_compression
	  << "))" << endl << ends;
    }
  else
    {
      ost << "(server_param "
	  << gwidth << " " 
	  << inertia_moment << " " 
	  << psize << " " 
	  << pdecay << " " 
	  << prand << " " 
	  << pweight << " " 
	  << pspeed_max << " " 
	  << paccel_max << " " 
	  << stamina_max << " " 
	  << stamina_inc << " " 
	  << recover_init << " " 
	  << recover_dthr << " " 
	  << recover_min << " " 
	  << recover_dec << " " 
	  << effort_init << " " 
	  << effort_dthr << " " 
	  << effort_min << " " 
	  << effort_dec << " " 
	  << effort_ithr << " " 
	  << effort_inc << " " 
	  << kick_rand << " "                             
	  << team_actuator_noise << " "                     
	  << prand_factor_l << " "                          
	  << prand_factor_r << " "                            
	  << kick_rand_factor_l << " "                       
	  << kick_rand_factor_r << " "                     
	  << bsize << " " 
	  << bdecay << " " 
	  << brand << " " 
	  << bweight << " " 
	  << bspeed_max << " " 
	  << baccel_max << " " 
	  << dprate << " " 
	  << kprate << " " 
	  << kmargin << " " 
	  << ctlradius << " " 
	  << ctlradius_width << " " 
	  << maxp << " "
	  << minp << " " 
	  << maxm << " " 
	  << minm << " " 
	  << maxnm << " " 
	  << minnm << " " 
	  << maxn << " " 
	  << minn << " " 
	  << Rad2Deg(visangle) << " " 
	  << visdist << " " 
	  << windir << " " 
	  << winforce << " " 
	  << winang << " " 
	  << winrand << " " 
	  << kickable_area << " " 
	  << catch_area_l << " " 
	  << catch_area_w << " " 
	  << catch_prob << " " 
	  << goalie_max_moves << " "                  
	  << ckmargin << " " 
	  << offside_area << " " 
	  << win_no << " " 
	  << win_random << " " 
	  << say_cnt_max << " " 
	  << SayCoachMsgSize << " " 
	  << clang_win_size << " "
	  << clang_define_win << " "
	  << clang_meta_win << " "
	  << clang_advice_win << " "
	  << clang_info_win << " "
	  << clang_mess_delay << " "
	  << clang_mess_per_cycle << " "
	  << half_time << " " 
	  << sim_st << " " 
	  << send_st << " " 
	  << recv_st << " " 
	  << sb_step << " " 
	  << lcm_st << " "                         
	  << SayMsgSize << " " 
	  << hear_max << " " 
	  << hear_inc << " " 
	  << hear_decay << " " 
	  << cban_cycle << " " 
	  << slow_down_factor << " "                           
	  << useoffside << " " 
	  << kickoffoffside << " " 
	  << offside_kick_margin << " " 
	  << audio_dist << " " 
	  << dist_qstep << " " 
	  << land_qstep << " " 
#ifdef NEW_QSTEP
	  << dir_qstep << " " 
	  << dist_qstep_l << " " 
	  << dist_qstep_r << " " 
	  << land_qstep_l << " " 
	  << land_qstep_r << " " 
	  << dir_qstep_l << " " 
	  << dir_qstep_r << " " 
#else
	  << -1 << " " 
	  << -1 << " " 
	  << -1 << " " 
	  << -1 << " " 
	  << -1 << " " 
	  << -1 << " " 
	  << -1 << " " 
#endif
	  << CoachMode << " " 
	  << CwRMode << " " 
	  << old_hear << " " 
	  << sv_st << " "

	  << start_goal_l << " "
	  << start_goal_r << " "

	  << fullstate_l << " "
	  << fullstate_r << " "

	  << drop_time;

      if ( ver >= 7.11 )
	{
	  ost << " " << slowness_on_top_for_left_team
	      << " " << slowness_on_top_for_right_team;
	}

      ost << ")" << endl << ends;
    }
}

