/* -*- Mode: C -*-
 *Header:
 *File: utility.C 
 *Author: Noda Itsuki
 *Date: 1995/02/24
 *EndHeader:
 */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */


#include <math.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include "param.h"
#include "types.h"
#include "utility.h"

/*
 *===================================================================
 *Part:	    Random number utility
 *===================================================================
 */
unsigned int randomize(void)
{
	static unsigned int l ;

	static struct timeval Tv ;
	static struct timezone Tz ;
	gettimeofday( &Tv , &Tz ) ;
	l = (unsigned long)(Tv.tv_usec) ;
	srandom( l ) ;
	return l ;
}


/*
 *===================================================================
 *Part:	    Angle
 *===================================================================
 */
double normalize_angle(double ang)
{
	while(1) {
		if (ang < -PI)
			ang += 2 * PI ;
		else if (ang > PI)
			ang -= 2 * PI ;
		else
			break ;
	}
	return ang ;
}


/*
 *==================================================================
 *Part:     Lowest Common Multiple
 *==================================================================
 */
int lcm (int a, int b)
{
  int tmp = 0, idx = 0, larger = max(a, b);
  do {
    idx++;
    tmp = larger * idx;
  } while (tmp % a != 0 || tmp % b != 0);
  return tmp;
}


/*
 *==================================================================
 *Part:     Tilde Expand
 *==================================================================
 */
// This function will return a copy of path_name with `~' replaced
// with the value of the `HOME' environment variable
// This means `~someone/foo' wont work.  Feel free to write
// a patch :) 
string tildeExpand ( const string& path_name )
{
  int tilde_pos = 0; 
  bool found = false;
  for ( int i = 0; i < path_name.length (); i++ )
    if ( path_name[i] == '~' )
      {
	tilde_pos = i;
	found = true;
	break;
      }
  
  string path = path_name;

  if ( found )
    {
      path.erase ( tilde_pos, tilde_pos+1 );
      path.insert ( tilde_pos, string ( getenv ( "HOME" ) ) );
    }
  return path;
}

